#include "SimpleWindow.h"

fkut_SimpleWindow::fkut_SimpleWindow(void)
{
	mainWin = new Fl_Window(512, 512, "FKUT Window");
	drawWin = new fk_Window(0, 0, 512, 512);

	mainWin->end();
	fk_Material::initDefault();

	light.setShape(&lightShape);
	light.setMaterial(White);
	light.getMaterial()->setAmbDiff(1.0, 1.0, 1.0);
	light.glMoveTo(0.0, 0.0, 0.0);
	light.glFocus(-1.0, -1.0, -1.0);

	scene.setBlendStatus(true);
	scene.entryCamera(&camera);
	scene.entryModel(&light);

	drawWin->setScene(&scene);

	setFPS(60);

	camera.glMoveTo(0.0, 0.0, 100.0);
	camera.glFocus(0.0, 0.0, 0.0);
	camera.glUpvec(0.0, 1.0, 0.0);

	tb = new fk_TrackBall(drawWin, &camera);
	tbFlag = false;

	return;
}

fkut_SimpleWindow::~fkut_SimpleWindow(void)
{
	drawWin->hide();
	mainWin->hide();

	delete drawWin;
	delete mainWin;
}

void fkut_SimpleWindow::setSize(int argW, int argH)
{
	mainWin->resize(mainWin->x(), mainWin->y(), argW, argH);
	drawWin->resizeWindow(0, 0, argW, argH);

	return;
}

void fkut_SimpleWindow::setBGColor(double r, double g, double b)
{
	scene.setBGColor((float)r, (float)g, (float)b);

	return;
}

void fkut_SimpleWindow::setFPS(int argFPS)
{
	if(argFPS == 0) {
		fps = 0;
		fps_admin.OnFrameSkip(false);
	} else {
		fps = argFPS;
		fps_admin.OnFrameSkip(true);
		fps_admin.SetFPS(argFPS);
	}

	return;
}

void fkut_SimpleWindow::setTrackBallMode(bool mode)
{
	tbFlag = mode;

	return;
}

void fkut_SimpleWindow::showGuide(fkut_GuideMode mode)
{
	guide.entryScene(&scene, mode);

	return;
}

void fkut_SimpleWindow::hideGuide(void)
{
	guide.removeScene(&scene);

	return;
}

void fkut_SimpleWindow::setCameraPos(double x, double y, double z)
{
	camera.glMoveTo(x, y, z);

	return;
}

void fkut_SimpleWindow::setCameraFocus(double x, double y, double z)
{
	camera.glFocus(x, y, z);
	tb->setLookTo(fk_Vector(x, y, z));

	return;
}

void fkut_SimpleWindow::entry(fk_Model &model)
{
	scene.entryModel(&model);
}

void fkut_SimpleWindow::remove(fk_Model &model)
{
	scene.removeModel(&model);
}

void fkut_SimpleWindow::open(void)
{
	mainWin->show();
	drawWin->show();

	return;
}

bool fkut_SimpleWindow::update(void)
{
	if(fps != 0) fps_admin.TimeRegular();

	if(mainWin->visible() == 0) {
		if(Fl::wait() == 0) {
			return false;
		} else {
			return true;
		}
	}

	if(fps_admin.GetDrawFlag() || fps == 0) drawWin->drawWindow();
	if(Fl::check() == 0) return false;

	if(tbFlag) tb->update();

	return true;
}
